// Allan Bowhill

#include "Config.h"


// Constructor
// set default values for config options

Config::Config():empRoot(DEFAULT_EMP_ROOT), configPath(DEFAULT_CONFIG)
	{
	}

// check for existence of config file at path

bool Config::checkEmpRoot(const char * path)
	{
	// establish an input file stream
	ifstream rootPath (path);

	// check for existence of path and return false for path not found 
	// or true for path found

	if (rootPath.fail())
		{
		rootPath.close();
		return false;
		}
	else 
		{
		rootPath.close();
		return true;
		}
	}

// Check for the existence of a config file in the named path.
// If file read fails, send error

bool Config::checkConfigFile(const char * path)
	{
	ifstream config (path);

	if(config.fail())
		{
		config.close();
		return false;
		}
	else
		{
		bool result = readConfigFile(config, path);
		config.close();
		return result;
		}
	}

// read config file
// sets empRoot if it is found in a file
// returns false if empRoot or file read failed
// returns true if empRoot was found

bool Config::readConfigFile(ifstream & config, const char * path)
	{
	// store the emp root key from the header file
	string key = EMP_ROOT_KEY;
	char buf[PATH_MAX] = {'\0'};

	while(!config.eof())
		{
		// get a line from file into buffer
		config.getline(buf, PATH_MAX);
		string testKey(buf);

		// if we have a cr in charpos 0 to 3 or
		// we have no positive identification of key
		// get another line

		if (testKey.rfind('\n') < 3 || testKey.find(key) == -1)
			continue;

		// we found the key
		// try to extract the value from the buffer
		// return false if file read failed for some reason

		bool result = getValue(testKey);
		config.close();
		return result;
		}

	// we did not find key in file
	return false;
	}

// get value from line in config that has matching key
// returns false if value was not large enough to be useful

bool Config::getValue(string & buf)
	{
	// since we found key, locate the starting and ending position 
	// of the pathname, which are both delimited by "
	// if they are not far enough away from each other to hold a
	// valid directory name, or have -1 as the position
	// return false
			
	int startVal = 0;
	int endVal = 0;

	startVal = buf.find('"');
	endVal = buf.rfind('"');

	if (endVal-startVal < 1)
		{
		cerr << "Error: Config file. Incorrect value between quotes" << endl;
		return false;
		}
	else
		{
		string final = buf.substr(startVal+1, endVal-startVal-1);
		empRoot = final.data();
		return true;
		}
	}


// returns root path or NULL if nothing found
const char * Config::getEmpRoot(void)
	{
	// since a config file is optional, the preferred method is to use 
	// the hard-coded value of DEFAULT_EMP_ROOT in the header file. If this
	// directory is found, use it. The hardcoded emp root has the fastest 
	// turnaround time for the program


	// check for the existence of the emp document root in preferred 
	// location PREF_EMP_ROOT

	if (checkEmpRoot(PREF_EMP_ROOT) == true)
		{
		empRoot = PREF_EMP_ROOT;
		return empRoot;
		}

	// if the emp document root was not found in the preferred location, 
	// check for the existence of it in the default location, which is 
	// the same directory as the emp executable

	else if (checkEmpRoot(DEFAULT_EMP_ROOT) == true)
		{
		empRoot = DEFAULT_EMP_ROOT;
		return empRoot;
		}

	// if no emp document root was found, check for the existence of a 
	// config file, which can contain the real location of emp root. 

	// if EMP_ROOT was not found, check for the existence of a config file 
	// in the preferred location PREF_CONFIG_PATH, then DEFAULT_CONFIG_PATH
	// if either is found

	else if	(checkConfigFile(PREF_CONFIG))
		{
		// check the root path to see if it exists
		// if it does, return it. If it doesn't exist, 
		// clear it, and return null

			if(checkEmpRoot(empRoot) == true)
				return empRoot;
			else
				{
				cerr << "emp root directory: [" << empRoot << "] not found" << endl;
				empRoot = NULL;
				return empRoot;
				}
		}

	else if (checkConfigFile(DEFAULT_CONFIG) == true)
			{
		// check the root path to see if it exists
		// if it does, return it. If it doesn't exist, 
		// clear it, and return null

			if(checkEmpRoot(empRoot) == true)
				return empRoot;
			else
				{
				cerr << "emp root directory: [" << empRoot << "] not found" << endl;
				empRoot = NULL;
				return empRoot;
				}
			}
		// the config file is not found in either location
		else
			return false;

		}

