// Allan Bowhill

#include "Link.h"

// constructor
//
// initialize attributes with proper XML entities

Link::Link(char * U, char * cT, char * sC) : Url(convertEntities(U)), clickText(convertEntities(cT)), shortDescr(convertEntities(sC))
	{
	}

// substitute any "&" with "&amp;" entity for xthml compatibility
// return modified string even if nothing changed.

char * Link::convertEntities(char * theString)
	{
	// buffer to hold modified string 
	char newString[1024] = { '\0' };
	// entity we will substitute for a lone ampersand
	const char newEntity[] = "&amp;";

	// scan input string for ampersand
	for (int ctr = 0; ctr < strlen(theString); ++ctr)
		{
		// we found an ampersand in input string
		// verify if it is part of an entity, or just a loner
		if (theString[ctr] == '&')
			{
			// if it's a loner,
			if (!isEntity(&theString[ctr]))
				{
				// place an "&amp;" entity in the output string
				strlcat(newString, newEntity, sizeof(newString));
				continue;
				}
			}
		// otherwise just transfer one char from input string to output string
		char charbuf[2] = {'\0'};
		charbuf[0] = theString[ctr];
		strlcat(newString, charbuf, sizeof(newString));
		}
	// copy whole output buffer to heap, so we can return it
	char * aString = new char[sizeof(newString)];
	strlcpy(aString, newString, sizeof(newString));
	return aString;
	}



// find out if substring begins an XML entity
// return true if position 0 is the start of an entity
// return false if it is a loner ampersand

bool Link::isEntity(const char * theString)
	{
	// by default, we assume this substring doesn't start with an entity
	bool isOne = false;

	// check substring after first char for 
	// matching ';' to indicate entity-ness

	for (int ctr = 1; ctr < strlen(theString); ++ctr)
		{
		// if we have reached ond-of-string
		// without finding semicolon, OR
		// we found another ampersand,
		// in which entities can't be nested,
		// return false right away, and don't
		// bother checking the rest of the string

		if (theString[ctr] == '\0' || theString[ctr] == '&')
			{
			isOne = false;
			break;
			}

		// we found the matching semicolon

		if (theString[ctr] == ';')
			{
			isOne = true;
			break;
			}
		}
	return isOne;
	}
