// Allan Bowhill

#include "LinkedList.h"

// constructors
LinkedList::LinkedList()
	{ 
	// setup dummy head node
	node * dummy = new (nothrow) node;
	assert (dummy);

	// set next node
	dummy->next = dummy;

	// set previous node
	dummy->prev = dummy;

	// setup head pointer
	head = dummy;

	// set head data to NULL
	head->item = NULL;
	}

LinkedList::~LinkedList()
	{
	delete [] head;
//	delete [] traversalPtr;
	}

// Print list
bool LinkedList::print()
	{
	LinkedList::assertBasic();
	if (LinkedList::empty())
		cout << "List empty" << endl;
	else
		{
 		cout << " (" << LinkedList::getSize() << ") : ";
		LinkedList::printItem(head->next);
 		cout << endl;
		}
	return true;
	}

// recursively travel list and print integer for tests
void LinkedList::printItem(node * current)
	{
	// cout << (char *)(current->item) << " ";
	cout << current->item << " ";

	// recurse
	if (current->next->item != NULL)
		{
		current = current->next;
		LinkedList::printItem(current);
		}
	}

// Other methods


// add a node to last (queue push)

bool LinkedList::addEnd (void * data)
	{
	LinkedList::assertBasic();
	// create an empty node
	node * newNode = new(nothrow) node;
	assert (newNode);

	// assign data to new node
	newNode->item = data;

	// point new node to last node
	newNode->prev = head->prev;

	// point last node to new node
	assert(head->prev->next);
	head->prev->next = newNode;

	// point new node to head
	newNode->next = head;

	// point head to new node
	head->prev = newNode;

	return true;
	}

// add a node to first (stack push)
bool LinkedList::addBegin(void * data)
	{
	LinkedList::assertBasic();

	// create an empty node
	node * newNode = new(nothrow) node;
	assert (newNode);

	// assign data to new node
	newNode->item = data;

	// point new node to first node
	newNode->next = head->next;

	// point last node to new node
	assert(head->next->prev);
	head->next->prev = newNode;

	// point new node to head
	newNode->prev = head;

	// point head to new node
	head->next = newNode;

	return true;
	}


// add a node at a specified index (vector push)

bool LinkedList::addIndex(int index, void * data)
	{
	LinkedList::assertBasic();

	// index is greater than zero
	assert(index >= 0);

	// Check if index is within bounds.
	// Return failure if list too small.
	if ((LinkedList::getSize()) < index) return false;

	// init the index node to remove
	// create an empty node
	node * newNode = new(nothrow) node;
	assert(newNode);

	// assign data to new node
	newNode->item = data;

	// get the index node to displace
	node * nodeToDisplace = head->next;

	// Search list for index insertion point.
	// Identify the node to be displaced to 
	// the right, if one exists.
	// Abort if any list element found is NULL.

	for (int i = 0; i < index; ++i)
		{
		assert(nodeToDisplace->item != NULL);
		nodeToDisplace = nodeToDisplace->next;
		}

	// link new node to left node
	nodeToDisplace->prev->next = newNode;

	// link left node to new node
	newNode->prev = nodeToDisplace->prev;

	// link new node to right node
	nodeToDisplace->prev = newNode;

	// link right node to newnode
	newNode->next = nodeToDisplace;

	LinkedList::assertBasic();
	return true;
	}

// delete last node (stack pop)
bool LinkedList::delEnd()
	{
	LinkedList::assertBasic();

	// test to see if list is empty
	if (LinkedList::empty()) return false;

	// link second to last node to head
	assert(head->prev->prev);
	head->prev = head->prev->prev;
	
	// delete last node
	assert(head->prev->next);
	delete head->prev->next;

	// link head to second to last node
	head->prev->next = head;

	return true;
	}

// delete first node (queue pop)
bool LinkedList::delBegin()
	{
	LinkedList::assertBasic();

	// test to see if list is empty
	if (LinkedList::empty()) return false;

	// link second node to head
	assert(head->next->next);
	head->next = head->next->next;

	// delete first node
	assert(head->next->prev);
	delete head->next->prev;

	// link head to second node
	head->next->prev = head;

	return true;
	}

// delete a node at an index (vector pop)
bool LinkedList::delIndex(int index)
	{
	LinkedList::assertBasic();

	// index is greater than zero
	assert(index >= 0);

	// Check if index is within bounds.
	// Return failure if list too small.
	if (LinkedList::getSize() < index) return false;

	// if list is empty, return failure
	if (LinkedList::empty()) return false;

	// init the index node to remove
	node * nodeToDelete = head->next;

	// Search list for index node to delete.
	// Last item stored in nodeToDelete is the node to delete.
	// Abort if any list element found is NULL.

	for (int i = 0; i < index; ++i)
		{
		assert(nodeToDelete->item != NULL);
		nodeToDelete = nodeToDelete->next;
		}

	// link right node to left node
	nodeToDelete->prev->next = nodeToDelete->next;

	// link left node to right node
	nodeToDelete->next->prev = nodeToDelete->prev;

	// set to null
	nodeToDelete->next =NULL;
	nodeToDelete->prev =NULL;
	nodeToDelete->item =NULL;
	
	// get the thing
	delete nodeToDelete;

	return true;
	}

// Peek at last node.
void * LinkedList::peekEnd()
	{
	LinkedList::assertBasic();

	// Return the last node.
	// If there is none, return head.

	if (head->prev->item != NULL)
		return head->next->item;
	else
		return head;
	}

// peek at first node
void * LinkedList::peekBegin()
	{
	LinkedList::assertBasic();

	// Return the top node.
	// If there is none, return head.

	if (head->next->item != NULL)
		return head->next->item;
	else
		return head;
	}


// peek at Index (vector peek)
void * LinkedList::peekIndex(int index)
	{
	LinkedList::assertBasic();

	// index is greater than zero
	assert(index >= 0);

	// Check if index is within bounds.
	// Return failure if list too small.
	if ((LinkedList::getSize()) < index) return NULL;

	// init node to peek with 0th element of list
	node * nodeToPeek = head->next;

	// Search list for index node to peek.
	// Last item stored in nodeToPeek is the node to peek.
	// Abort if any list element found is NULL.

	for (int i = 0; i < index; ++i)
		{
		assert(nodeToPeek->item != NULL);
		nodeToPeek = nodeToPeek->next;
		}

	// return node
	return nodeToPeek->item;
	}

// sort list in ascending order
// takes pointer tocomparison function

bool LinkedList::sort(int(*comp)(Citem, Citem))
	{
	LinkedList::assertBasic();

	// return size based on boolean if zero
	if (LinkedList::empty())
		return false;

	// get array of linked list (with null pointer at end)
	void ** array = LinkedList::copyToArray();
	assert(array != NULL);

	// get size of list
	int size = getSize();

	// sort items in the list
	qsort(array, size, sizeof(head->next->item), comp);

	// purge our original linked list
	assert(LinkedList::clear());
	
	// copy back items from array to linked list
	for(int i = 0; i < size; ++i)
		assert(addBegin(array[i]));

	// destroy array now that we have used it
	delete array;

	return true;
	}



void ** LinkedList::copyToArray()
	{
	LinkedList::assertBasic();

	// return size based on boolean if zero
	if (LinkedList::empty())
		return NULL;

	// get size of list
	int size = getSize();

	// setup void * array to copy node items
	void ** array = new (nothrow) void * [(size+1)];
	assert(array);
	
	// initialize a work node to head
	node * our = head;

	// copy linked list items to array
	for(int i = 0; i < size; ++i)
		{
		// we have a next item
		assert(our->next->item);

		// pass it into the array
		array[i] = our->next->item;

		// transpose
		our = our->next;
		}

	// add a null to last char in array
	array[(size)] = '\0';

	// return array pointer
	return array;
	}


// reset traveral pointer for traversals
void LinkedList::resetTraversalPtr()
	{
	traversalPtr = head;
	}

// traverse to the next node, returning the item
void * LinkedList::getNextElement()
	{
	traversalPtr = traversalPtr->next;
	return traversalPtr->item;
	}



// get size of linked list
int LinkedList::getSize()
	{
	LinkedList::assertBasic();

	// return size based on boolean if zero
	if (LinkedList::empty())
		return 0;

	// counter is number of links
	int counter = 0;

	// this is a work node
	node * our = head;

	// iterate through list until head
	// incrementing counter along the way

	while (our->next->item != NULL)
		{
		++counter;
		our = our->next;
		}

	// we are on last node in list
	// therefore we are head's prev

	assert(head->prev->item == our->item);

	return counter;
	}

// check for empty list
bool LinkedList::empty()
	{
	LinkedList::assertBasic();

	if ((head->next->item == NULL) && (head->prev->item == NULL))
		return true;
	else
		return false;
	}

// clear list
bool LinkedList::clear()
	{
	LinkedList::assertBasic();

	// if list is empty return false
	if (LinkedList::empty())
		return false;

	// while we can delete nodes, do it

	else
	while(LinkedList::delBegin() == true)
		continue;

	// list must be empty afterwards
	assert(LinkedList::empty());

	return true;
	}

// make basic assumptions
void LinkedList::assertBasic()
	{
	// Pointers in dummy always point to something,
	// even the dummy itself (e.g. pointers are never nulls).

	assert(head->next && head->prev);

	// Dummy either points to itself with both pointers (list empty)
	// or both pointers point outside the dummy (list not empty).
	// Dummy nver has just a single pointer pointing to itself.

	assert(((head->next == head) && (head->prev == head)) || ((head->next != head) && (head->prev != head)));

	// Dummy node can not have been modified inadvertently.
	assert(head->item == NULL);
	}
