// Allan Bowhill

#ifndef LINKEDLIST_H
#define LINKEDLIST_H

#include "../main.h"
#include <iostream>
#include <new>
#include <cstdlib>
//#include <fstream>
#include <cassert>
#include <string>

typedef void * voidPtr;

using namespace std; 

// Class: LinkedList
//
// Abstract Class
//
// Remarks:
// Doubly-linked list, circular


class LinkedList
	{
	protected:

		// constructors
		LinkedList();

		// destructor
		virtual ~LinkedList();

		// functions

		// add a node to last (queue push)
		virtual bool addEnd(void *);

		// add a node to first (stack push)
		virtual bool addBegin(void *);

		// add a node at a specified index (vector push)
		virtual bool addIndex(int, void *);

		// delete last node (stack pop)
		virtual bool delEnd();

		// delete first node (queue pop)
		virtual bool delBegin();

		// delete a node at an index (vector pop)
		virtual bool delIndex(int);

		// peek at last node (stack peek)
		virtual void * peekEnd();

		// peek at first node
		virtual void * peekBegin();

		// peek at Index (vector peek)
		virtual void * peekIndex(int);

		// sort list in ascending order
		// via pointer to comparison function
		virtual bool sort(int(*comp)(Citem, Citem));
	
		// copy to array pointer on heap
		// destroy after use
		virtual void ** copyToArray();

		virtual void resetTraversalPtr();
		virtual void * getNextElement();

		// search list for size
		virtual int getSize();

		// check for empty
		virtual bool empty();

		// clear list
		virtual bool clear();
		
		// print list
		virtual bool print();
	


	private:
		// copy constructor
		LinkedList(const LinkedList & aLinkedList) { }

		// variables

		// node structure
		struct node
			{
			void * item;
			node * next;
			node * prev;
			};

		// pointer to head node
		node * head;

		// Item to print
		virtual void printItem(node *);

		node * traversalPtr;

		// basic assertion
		virtual void assertBasic();

	};

#endif
