#ifndef __TESTPARAMETER_H
#define __TESTPARAMETER_H

#define GOOD_QUERY_STRING "q=123.456.789"
#define WRONG_VAR_1 "r=123.456.789"
#define WRONG_VAR_2 "^^^^^=123.456.789"
#define MISSING_VAR "=123.456.789"
#define MISSING_EQ "123.456.789"
#define TOO_MANY_ZEROS "q=012300.0045600.078900000"
#define TOO_MUCH_JUNK_1 "q=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx012300.0045600.078900000"
#define TOO_MUCH_JUNK_2 "q=aaddi-0j3jfp9u-9bnbu-ugijvoaigu9ybhuhjj-f0E98R0WHGVJFPOIFDJFkpgu9ugoij0wrr0-werjggogh-]8=4i29j;gjgj[s0gipsjgdpijsofdf fjsldjmsidhf09wut9gnigu]W[0FI-OI0FI]-JWOJT'P0gi]05u]-9utj'poorfOJRGEPTGIEKTOSIF]'[GIKZD'GOJIOEAFP"
#define LESS_THAN_MIN_LENGTH_QS "q=1"
#define MIN_LENGTH_QS "q=123.456.789"



#include <iostream>
#include "Parameter.h"
#include <cxxtest/TestSuite.h>


class ParamTest: public CxxTest::TestSuite
	{
	public:

	// perform this unconditionally
	void test_setEnvironment()
		{
		TS_ASSERT(setenv(QUERY_STRING,GOOD_QUERY_STRING,1) == 0);
		TS_ASSERT(setenv(SERVER_NAME,"MyServer",1) == 0);
		TS_ASSERT(setenv(SERVER_ADDR,"10.0.0.3",1) == 0);
		TS_ASSERT(setenv(REMOTE_ADDR,"10.0.0.1",1) == 0);
		TS_ASSERT(setenv(SCRIPT_NAME,"AN_EMP.cgi",1) == 0);
		TS_ASSERT(setenv(HTTP_USER_AGENT,"Mozilla",1) == 0);
		}

	// TEST: getQueryString
	void test_getQueryString()
		{
		Parameter aParameter;
		char aString[] = GOOD_QUERY_STRING;
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getQueryString()), String(aString));
		}

	// TEST: getHttpUserAgent
	void test_getHttpUserAgent()
		{
		Parameter aParameter;
		char aString[] = "Mozilla";
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getHttpUserAgent()), String(aString));
		}

	// TEST: getRemoteAddress
	void test_getRemoteAddr()
		{
		Parameter aParameter;
		char aString[] = "10.0.0.1";
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getRemoteAddr()), String(aString));
		}

	// TEST: getServerName
	void test_getServerName()
		{
		Parameter aParameter;
		char aString[] = "MyServer";
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getServerName()), String(aString));
		}

	// TEST: getServerAddr
	void test_getServerAddr()
		{
		Parameter aParameter;
		char aString[] = "10.0.0.3";
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getServerAddr()), String(aString));
		}

	// TEST: getScriptName
	void test_getScriptName()
		{
		Parameter aParameter;
		char aString[] = "AN_EMP.cgi";
		typedef CXXTEST_STD(string) String;
		TS_ASSERT_EQUALS(String(aParameter.getScriptName()), String(aString));
		}

	// TEST: check good query string
	void test_check_good_qs()
		{
		TS_ASSERT(setenv(QUERY_STRING, GOOD_QUERY_STRING, 1) == 0);
		Parameter aParameter;
		TS_ASSERT_EQUALS(aParameter.check(), true);
		}

	// test for wrong CGI variable
	void test_check_wrong_var_qs()
		{
		TS_ASSERT(setenv(QUERY_STRING, WRONG_VAR_1 ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check wrong variable, multiples
	void test_check_wrong_other_qs()
		{
		TS_ASSERT(setenv(QUERY_STRING, WRONG_VAR_2 ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check missing variable  
	void test_check_missing_variable_qs()
		{
		TS_ASSERT(setenv(QUERY_STRING, MISSING_VAR ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check missing equal sign in query string
	void test_check_missing_equals_qs()
		{
		TS_ASSERT(setenv(QUERY_STRING, MISSING_EQ ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check junk string 1
	void test_check_junk_1()
		{
		TS_ASSERT(setenv(QUERY_STRING, TOO_MUCH_JUNK_1 ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check junk string 2
	void test_check_junk_2()
		{
		TS_ASSERT(setenv(QUERY_STRING, TOO_MUCH_JUNK_2 ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check less than minimum length
	void test_less_than_min_length()
		{
		TS_ASSERT(setenv(QUERY_STRING, LESS_THAN_MIN_LENGTH_QS ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_DIFFERS(aParameter.check(), true);
		}

	// check minimal length string
	void test_min_length_qs()
		{
		unsetenv(QUERY_STRING);
		TS_ASSERT(setenv(QUERY_STRING, MIN_LENGTH_QS ,1) == 0);
		Parameter aParameter;
		TS_ASSERT_EQUALS(aParameter.check(), true);
		}


	// TEST: getParameterField
	void test_getParameterField()
		{
		TS_ASSERT(setenv(QUERY_STRING, GOOD_QUERY_STRING ,1) == 0);
		Parameter aParameter, bParameter;
		aParameter.check();

		TS_ASSERT_EQUALS(*(aParameter.getParameterField()),123);
		TS_ASSERT_EQUALS(*(aParameter.getParameterField()),456);
		TS_ASSERT_EQUALS(*(aParameter.getParameterField()),789);
		TS_ASSERT_EQUALS(*(aParameter.getParameterField()),0);

		// test for elimination of extra zeros appended to each field
		TS_ASSERT(setenv(QUERY_STRING, TOO_MANY_ZEROS ,1) == 0);
		bParameter.check();
		TS_ASSERT_EQUALS(*(bParameter.getParameterField()),123);
		TS_ASSERT_EQUALS(*(bParameter.getParameterField()),456);
		TS_ASSERT_EQUALS(*(bParameter.getParameterField()),789);
		TS_ASSERT_EQUALS(*(bParameter.getParameterField()),0);
		}
	};

#endif
